/*
 * Copyright (c) 2018 One Hill Technologies, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const { PropertyDescriptor } = require ('base-object');
const NamespaceDescriptor = require ('./namespace-descriptor');

/**
 * A descriptor for binding a Socket.IO socket to a property.
 */
class SocketDescriptor extends PropertyDescriptor {
  constructor (connection) {
    super ();

    this.connection = connection;
  }

  defineProperty (obj, name) {
    const connectionName = this.connection || name;

    Object.defineProperty (obj, name, {
      get () {
        return this.app.lookup ('service:io').connection (connectionName);
      }
    });
  }

  of (nsp) {
    return new NamespaceDescriptor (this, nsp);
  }
}

function io (connection) {
  return new SocketDescriptor (connection);
}

module.exports = io;


